#JWST Dictionary
# Version 3.3, Modified 7/20/2015

# This software is provided as is without any warranty whatsoever.
# Permission to use and modify is granted. See associated License.

#For now use the standard window manager in Python
import Tkinter
from Tkinter import *
import textwrap
import re
import time
import os
import tkFont
import textwrap
import urllib2

# Linux or Mac?
global whatisOS;
whatisOS = os.popen("uname").read().rstrip()

# My personal version or others?
# My version keeps add/edit/delete entry capability
global mine; mine = False

# Version variables (to check for updates)
global version; version = 3.3
global updateDate; updateDate = 'July 19, 2015'

#%%%%%%%%%%%%%%%%%%%%%%%%
# --VERIFY PATH CLASS-- %
#%%%%%%%%%%%%%%%%%%%%%%%%

# Class to check path. It's of the Tk class so that an error window can
# be popped up if the path is incorrect or the file does not exist.
class CheckPath(Tkinter.Tk):
    def __init__(self,parent):
        # Look for path to acronyms.dat file with environment variable
        self.datafilepath = os.popen("echo $JWSTACROPATH").read().rstrip()
        # Set size of window.
        self.xsize="420"
        self.ysize="170"

        # If environment variable does not exist:
        if self.datafilepath == '':
            if whatisOS == 'Darwin': 
                # Set default path for Mac
                self.datafilepath='/Applications/JWST.Acronym.Dictionary.app' \
                  +'/Contents/Resources/'
            # If not mac, then no default, raise error and quit
            else:
                # Set default to current path
                self.datafilepath = os.popen("pwd").read().rstrip()+'/'
                
        # Okay, so it exists, now verify location of acronyms.dat:
        if os.path.isfile(self.datafilepath+"acronyms.dat"):
            # IF true, then start program and send path to it
            self.start()
        else:
            # IF false, then raise error and quit when window closes
            self.errorwindow()

    # Error window to pop up      
    def errorwindow(self):

        # Set up window
        Tkinter.Tk.__init__(self)
        self.title("ERROR")
        self.geometry("%sx%s" % (self.xsize,self.ysize))
        self.configure(relief=RAISED)
        self.configure(background="Gainsboro")
        
        # Set up fonts
        if whatisOS == 'Darwin':
            self.ErrorFont1 = tkFont.Font(family="system", size=13)
            self.ErrorFont2 = tkFont.Font(family="Verdana", size=12)
        else:
            self.ErrorFont1 = tkFont.Font(family="system", size=11)
            self.ErrorFont2 = tkFont.Font(family="Verdana", size=10)

        # Decorative frame
        decframe = Frame(self,relief=RAISED,border=2,bg='Gainsboro')

        # Add OK button
        okb = Button(self, text="OK", command=self.quit,bg="Gainsboro",fg="Black" \
                        ,font=self.ErrorFont1,borderwidth=1 \
                        ,highlightcolor="White",highlightbackground="Gainsboro" \
                        ,highlightthickness=1,width=7,state=NORMAL,relief=RAISED)
                        
        # Set info (cannot break the string up to make code
        # neater because sprintf doesn't like it)
        errorinfo = "Cannot find acronyms.dat in %s \n\nVerify the environment variable, JWSTACROPATH." % self.datafilepath

        # Text widget
        errortext = Text(self,width=45,wrap=WORD,relief=FLAT,borderwidth=0 \
                        ,highlightthickness=0,background="Gainsboro" \
                        ,foreground="Black",height=6,font=self.ErrorFont2)
        errortext.insert(0.0,errorinfo)
        errortext.config(state=DISABLED)

        # Place things in window
        decframe.place(x=0,y=0,width=int(self.xsize),height=int(self.ysize))
        okb.place(x=int(self.xsize)-100,y=int(self.ysize)-40)
        errortext.place(x=20,y=20,width=int(self.xsize)-40)

        # Start the infinite loop
        self.mainloop()

    # If acronym file is found then start gui.
    def start(self):
        # Call acrodictapp with no parent (None)
        app = AcroDictApp(None,self.datafilepath)
        # Start the infinite loop
        app.mainloop()

######################
# Main program (GUI) #
######################        
# Defining AcroDictApp to be the Tk class
class AcroDictApp(Tkinter.Tk):

    # Initialization of window.
    def __init__(self,parent,acrofilepath):
        # Tk initialization
        Tkinter.Tk.__init__(self,parent)
        
        # Keeping track of parent
        self.parent = parent
        # Window title
        self.title('JWST Acronym Dictionary')
        # Set window size, color, and general look
        self.geometry("340x640")
        self.configure(relief=RAISED)
        self.configure(background="SlateGrey")
        # Set global history variable
        self.history = []
        # Set up link exceptions variable (acronyms with mixed case)
        # Need to find someway around this in version 4.
        self.exceptions =['NIRCam','NIRSpec']
        # Set path to acronym file (verified in above class)
        self.datafilepath=acrofilepath        
        # Transparency and background image? (Mac only)
        if whatisOS == 'Darwin':
            # Verify exists only if Mac
            if os.path.isfile(self.datafilepath+'JAD_background.gif'):
                self.backcolor="systemTransparent"
            # If can't find, set background = white
            else:
                print "Didn't find JAD_background.gif in "+self.datafilepath+'.'
                print "Setting background color to white."
                self.backcolor="white"
        # If the OS is Linux or other, just set to white.
        else:
            self.backcolor="white"
            
        # If running in terminal, tell user where looking for acronyms.dat
        print 'Looking for acronyms.dat in '+self.datafilepath

        # MENUBAR (will appear at top of desktop for mac)
        top = Menu(self)                             # Make menu object for main
        self.config(menu=top)                        # cross-link window to menu
        filemenu = Menu(top)                         # Make submenu object
        # attach options to submenu
        filemenu.add_command(label='Sync Acronyms',command=self.SyncAcros)
        filemenu.add_command(label='Exit',command=sys.exit)    
        # attach submenu as a cascade menu item to main menu 
        top.add_cascade(label='File',menu=filemenu) 
        # Repeat for edit submenu
        editmenu = Menu(top)                         
        editmenu.add_command(label='Add Entry',command=self.OnAddClick)
        editmenu.add_command(label='Delete Entry',command=self.OnDeleteClick)
        editmenu.add_command(label='Edit Entry',command=self.OnEditClick)
        global links_onoff; links_onoff = BooleanVar(); links_onoff.set(True)
        editmenu.add_checkbutton(label='Links',onvalue=1,offvalue=0,variable=links_onoff)
        global highlight_onoff; highlight_onoff = BooleanVar(); highlight_onoff.set(True)
        editmenu.add_checkbutton(label='Highlight Matches',onvalue=1,offvalue=0 \
                                 ,variable=highlight_onoff)
        # Disable if not my version
        if mine == False:
            editmenu.entryconfig("Add Entry",state="disabled")
            editmenu.entryconfig("Delete Entry",state="disabled")
            editmenu.entryconfig("Edit Entry",state="disabled")     
        top.add_cascade(label='Edit', menu=editmenu) 
        # Repeat for help submenu
        helpmenu = Menu(top)
        helpmenu.add_command(label='About',command=self.About)
        helpmenu.add_command(label='View README',command=self.readme)
        helpmenu.add_command(label='Check for Latest Version',command=self.CheckUpdate)
        top.add_cascade(label='Help',menu=helpmenu)
        
        #Initialize
        self.initialize()

        #----------------------#
        # Tags to Canvas Items #
        #----------------------#
        # have to put it here for it to work. 
        self.db.tag_bind('dbTag','<Button-1>',self.onDbClick)
        self.gbb.tag_bind('gbbTag','<Button-1>',self.onGbbClick)
        self.gfb.tag_bind('gfbTag','<Button-1>',self.onGfbClick)

        
    #Initializing stuff
    def initialize(self):

        #-------#
        # Fonts #
        #-------#
        # Different fonts needed for Mac and Linux.
        if whatisOS == 'Darwin':
            # Normal result title font
            self.customFont1 = tkFont.Font(family="Times", size=18 \
                                        ,weight="bold")
            # Listbox -> Matches font
            self.customFont2 = tkFont.Font(family="Plantagenet Cherokee", size=14)
            # Grey instructions title font
            self.instructFont1 = tkFont.Font(family="Times",size=18 \
                                         ,weight="bold")
            # Grey instructions details font
            self.instructFont2 = tkFont.Font(family="Times",size=16)

            # About window fonts
            self.AboutFont1 = tkFont.Font(family="system", size=13)
            self.AboutFont2 = tkFont.Font(family="Verdana", size=12)
            self.AboutFont3 = tkFont.Font(family="Verdana", size=11)
            
            # Readme window fonts
            self.ReadFont1 = tkFont.Font(family="system", size=13)
            self.ReadFont2 = tkFont.Font(family="Verdana", size=12)
            self.ReadFont3 = tkFont.Font(family="Verdana", size=11)

            # Update window fonts
            self.UpdateFont1 = tkFont.Font(family="system", size=13)
            self.UpdateFont2 = tkFont.Font(family="Verdana", size=13)
        else:
            # Normal title font
            self.customFont1 = tkFont.Font(family="Nimbus Roman No9 L", size=14 \
                               , weight="bold")
            # Listbox -> Matches font
            self.customFont2 = tkFont.Font(family="DejaVu Serif", size=10 \
                                           ,weight="bold")
            # Grey instructions title font
            self.instructFont1 = tkFont.Font(family="Nimbus Roman No9 L",size=14 \
                                         , weight="bold")
            # Grey instructions details font
            self.instructFont2 = tkFont.Font(family="Nimbus Roman No9 L" \
                                             ,size=12,weight="bold")

            # About window fonts
            self.AboutFont1 = tkFont.Font(family="system", size=11)
            self.AboutFont2 = tkFont.Font(family="Verdana", size=9)
            self.AboutFont3 = tkFont.Font(family="Verdana", size=9)
            
            # Readme window fonts
            self.ReadFont1 = tkFont.Font(family="system", size=11)
            self.ReadFont2 = tkFont.Font(family="Verdana", size=9)
            self.ReadFont3 = tkFont.Font(family="Verdana", size=9)
            
            # Update window fonts
            self.UpdateFont1 = tkFont.Font(family="system", size=11)
            self.UpdateFont2 = tkFont.Font(family="Verdana", size=10)

        #-------------------------------#
        # Definition and Details window #
        #-------------------------------#
        # Make image background of label and use label as container
        if self.backcolor == "systemTransparent":
            self.tkimage = PhotoImage(file=self.datafilepath+'JAD_background.gif') 
            self.defdetframe=Label(self,image = self.tkimage)
            self.defdetframe.image=self.tkimage
        # If not, make a big white label for container
        else:
            self.defdetframe=Label(self,background=self.backcolor,width=340,height=300)
        # Place label (which I'm calling a frame, but it's not a true frame)
        self.defdetframe.place(x=0,y=40,width=340,height=300)
       
        # Match title and tracking variable
        global labelVariable; labelVariable = Tkinter.StringVar()
        # Make text object (BetterText is a modified Text Class; defined below)
        self.definText = BetterText(self.defdetframe,width=45,wrap=WORD,relief=FLAT \
                          ,borderwidth=0,highlightthickness=0 \
                          ,background=self.backcolor,font=self.customFont1,height=2)
        self.definText.configure(state='normal')
        # Place
        self.definText.place(x=10,y=10,width=320,height=65)

        # Match result box
        self.detailText = BetterText(self.defdetframe,width=45,wrap=WORD,relief=FLAT \
                               ,borderwidth=0,highlightthickness=0 \
                               ,background=self.backcolor,font=self.instructFont2)
        self.detailText.configure(state='normal')
        self.detailText.place(x=10,y=80,width=320,height=200)

        # Highlight tags
        self.detailText.tag_config('highlight',background='yellow')
        self.definText.tag_config('highlight',background='yellow')

        # Link tags
        self.detailText.tag_config('hyper',foreground="blue",underline=1)
        self.detailText.tag_bind('hyper','<Enter>' \
                                 ,self.detailText.config(cursor="hand2"))
        self.detailText.tag_bind('hyper','<Leave>',self.detailText.config(cursor=""))
        self.detailText.tag_bind('hyper','<Button-1>',self.textClick)
        
        # Instructions box to display when entry object is empty.
        instructionsText1 = 'Type any letter to start an acronym search.\n\n'
        instructionsText2 = 'Insert a % before the first letter to search by keyword.'
        self.instructTextBox = BetterText(self.defdetframe,width=31,wrap=WORD \
                                          ,relief=FLAT \
                                    ,borderwidth=20,highlightthickness=0 \
                                    ,background=self.backcolor,foreground="SlateGrey" \
                                    ,font=self.instructFont1)
        self.instructTextBox.insert(0.0,instructionsText1)
        self.instructTextBox.insert(END,instructionsText2)
        self.instructTextBox.tag_config('diffFont',font=self.instructFont2)
        self.instructTextBox.tag_add('diffFont','3.0',END)
        self.instructTextBox.configure(state=DISABLED)

        #-----------#
        # Searchbar #
        #-----------#  
        # Searchbar must be put here so that background image displays right away.
        # (Searchbar class is defined below)
        self.search = SearchBar(0,0,340,40)
        # Place autocomplete entry box on top of search bar canvas object.
        # (AutocompleteEntry class defined below)
        self.entry2 = AutocompleteEntry(self,width=30,relief=FLAT,borderwidth=0 \
                                        ,highlightthickness=0,fg="grey" \
                                        ,background="white")
        self.entry2.insert(0,'>SEARCH')
        self.entry2.bind("<FocusIn>",self.searchDefaultTextRemove)
        # Place
        self.entry2.place(x=80,y=10,width=230,height=20)
        # Delete entry button/shape (canvas object with tag)
        self.db = Canvas(width=16, height=16,borderwidth=0,highlightthickness=0 \
                         ,background="white")   
        self.db.create_oval(0,0,14,14,fill="grey",outline="grey",tags='dbTag')
        if whatisOS == 'Darwin':
            self.db.create_line(4,4,11,11,fill="white",width=2,tags='dbTag')
            self.db.create_line(4,11,11,4,fill="white",width=2,tags='dbTag')
        else:
            self.db.create_line(3,3,11,11,fill="white",width=2,tags='dbTag')
            self.db.create_line(3,11,11,3,fill="white",width=2,tags='dbTag')

        #-----------------#
        # History Buttons #
        #-----------------#
        # Search history Go back button  
        self.gbb = Canvas(width=16,height=16,borderwidth=0,highlightthickness=0 \
                          ,background="#778899")
        # Keep track, so can disable/able later (may or may not need to keep track)
        self.bk = self.gbb.create_polygon(2,8,14,2,14,14,fill="black",tags='gbbTag')
        # Search history Go forward button
        self.gfb = Canvas(width=16,height=16,borderwidth=0,highlightthickness=0 \
                          ,background="#778899")
        # Keep track, so can disable/able later (may or may not need to keep track)
        self.fw = self.gfb.create_polygon(2,2,14,8,2,14,fill="black",tags='gfbTag')

        #-----------------#
        # Results Listbox #
        #-----------------#
        # Make frame to hold everything
        self.resBoxframe = Frame(self,width=340,height=300,background='#778899')
        # Place
        self.resBoxframe.place(x=0,y=340,width=340,height=280)
        # Results listbox has different color for Linux/Mac
        if whatisOS == 'Darwin':
            self.resBox = Listbox(self.resBoxframe,width=42,height=260,relief=FLAT \
                    ,bg='LightGrey',borderwidth=0,highlightthickness=0 \
                    ,font=self.customFont2)
        else:
            # Linux Listbox font is too dark. Need to lighten.
            self.resBox = Listbox(self.resBoxframe,width=42,height=260,relief=FLAT \
                              ,bg='LightGrey',borderwidth=0,highlightthickness=0 \
                              ,font=self.customFont2,foreground="gray16")
        # Add scrollbar and configure to Results listbox
        self.scrollbar = Tkinter.Scrollbar(self,command=self.resBox.yview)
        self.resBox.config(yscrollcommand=self.scrollbar.set)
        self.resBox.config(selectbackground='LightSlateGrey',selectforeground='White')
        # Bind selection of result to command
        self.resBox.bind("<<ListboxSelect>>", self.entry2.selection)
        self.resBox.bind("<Return>", self.entry2.selection)
        # Pack into frame
        self.resBox.pack(padx=0,pady=2)   
        # More bindings of resBox selection for reset of history     
        self.resBox.bind("<Button-1>",self.hideButtons)
        self.resBox.bind("<Return>",self.hideButtons)
        self.resBox.bind("<Up>",self.hideButtons)
        self.resBox.bind("<Down>",self.hideButtons)

        
        #---------------------#
        # Populate dictionary #
        #---------------------#
        self.PopAcroDict()
        
        #------------------#
        # Entries info box #
        #------------------#
        self.nEntries = Entry(self,width=31,relief=FLAT,borderwidth=0 \
                                   ,background="SlateGrey",foreground="White" \
                                   ,highlightthickness=0)
        self.nEntries.place(x=10,y=620,width=320,height=20)
        # Populate entries object at bottom of gui
        self.NumEntries()

        
    #########################
    # HELP MENU DEFINITIONS #
    #########################
    
    # The About window
    def About(self):
        
        # Read in License file (and remove \r)
        with open(self.datafilepath+'JAD.License') as f:
            LicenseText = "".join(f.read().replace('\r','').splitlines(1)[0:])
        f.close()
        # Strip out all bad escape characters (i.e., \r)
        
        
        # Create top window
        AboutWin = Toplevel()
        AboutWin.geometry("400x600")
        AboutWin.configure(background="Gainsboro")
        AboutWin.title("About")

        # Create Frame
        aboutframe = Frame(AboutWin,relief=FLAT,bg="Gainsboro")

        # Create button to close window
        closeb = Button(AboutWin, text="CLOSE", command=AboutWin.destroy \
                            ,bg="Gainsboro",fg="Black" \
                   ,font=self.AboutFont1,borderwidth=1 \
                   ,relief=RAISED,width=7,state=NORMAL,highlightthickness=1 \
                            ,highlightbackground='Gainsboro' \
                            ,highlightcolor='White')

        # Frame to hold text and scrollbar
        textframe = Frame(aboutframe,relief=RIDGE,bg='White',borderwidth=2)
        # Create text window to display readme
        abouttext = Text(textframe,width=45,relief=FLAT \
                        ,borderwidth=0,highlightthickness=0 \
                        ,background="White",foreground="Black" \
                        ,height=22,font=self.AboutFont2,wrap=WORD)
        abouttext.insert(0.0,LicenseText)
        abouttext.config(state=DISABLED)
        # Create scrollbar and configure for text window
        aboutscr = Scrollbar(textframe)
        aboutscr.config(command=abouttext.yview)
        abouttext.config(yscrollcommand=aboutscr.set)
        # Pack scrollbar and text object into frame
        aboutscr.pack(side="right",fill="y",expand=False)
        abouttext.pack(side="left",fill="both",expand=True)
        
        # JWST icon for top of window
        self.iconimage = PhotoImage(file=self.datafilepath+'JWST_acro.gif',format='gif') 
        self.abouticon_label =Tkinter.Label(aboutframe,image = self.iconimage)
        self.abouticon_label.image=self.iconimage

        # Title Label
        aboutinfo = "\nJWST Acronym Dictionary\nVersion %2.1f\n\nCreated by Daniel Kiminki, University of Arizona, 2014\n\nThis software is provided as is without any warranty." % (version)

        aboutlabel = Label(aboutframe,bg='Gainsboro',fg='Black',font=self.AboutFont2 \
                          ,text=aboutinfo,anchor=CENTER)

        # Copyright label
        copylabel = Label(aboutframe,bg='Gainsboro',fg='Black',font=self.AboutFont3 \
                          ,text='Copyright (c) 2014 Daniel Kiminki',anchor=CENTER)

        # Place top frame.        
        aboutframe.place(x=0,y=0,width=600,height=600)

        # Place everything else
        self.abouticon_label.place(x=200,y=50,anchor=CENTER)
        aboutlabel.place(x=0,y=85,width=400)
        textframe.place(x=0,y=250,width=400,height=250)
        copylabel.place(x=0,y=520,width=400)
        closeb.place(x=280,y=560)
        

    # Readme window
    def readme(self):

        # Read in README file (skipping first 3 lines)
        # (also removing \r)
        with open(self.datafilepath+'README.dictionary') as f:
            ReadmeText = "".join(f.read().replace('\r','').splitlines(1)[3:])
        f.close()

        # Create top window
        ReadmeWin = Toplevel()
        ReadmeWin.geometry("500x600")
        ReadmeWin.configure(background="Gainsboro")
        ReadmeWin.title("")

        # Create Frame
        readframe = Frame(ReadmeWin,relief=FLAT,bg="Gainsboro")

        # Create button to close window
        closeb = Button(ReadmeWin, text="CLOSE", command=ReadmeWin.destroy \
                   ,bg="Gainsboro",fg="Black" \
                   ,font=self.ReadFont1,borderwidth=1 \
                   ,relief=RAISED,width=7,state=NORMAL,highlightthickness=1 \
                            ,highlightbackground='Gainsboro' \
                            ,highlightcolor='White')

        # Frame to hold text and scrollbar
        textframe = Frame(readframe,relief=RIDGE,bg='White',borderwidth=2)
        # Create text window to display readme
        readtext = Text(textframe,width=45,relief=FLAT \
                        ,borderwidth=0,highlightthickness=0 \
                        ,background="White",foreground="Black" \
                        ,height=230,font=self.ReadFont2)
        readtext.insert(0.0,ReadmeText)
        readtext.config(state=DISABLED)
        # Make scrollbar and configure for text object
        readscr = Scrollbar(textframe)
        readscr.config(command=readtext.yview)
        readtext.config(yscrollcommand=readscr.set)
        # Pack scrollbar and text object into frame
        readscr.pack(side="right",fill="y",expand=False)
        readtext.pack(side="left",fill="both",expand=True)
        
        # JWST icon
        self.iconimage = PhotoImage(file=self.datafilepath+'JWST_acro.gif' \
                                    ,format='gif') 
        self.helpicon_label =Tkinter.Label(readframe,image = self.iconimage)
        self.helpicon_label.image=self.iconimage

        # Title Label
        readlabel = Label(readframe,bg='Gainsboro',fg='Black',font=self.ReadFont2 \
                          ,text='README for JWST Acronym Dictionary',anchor=CENTER)

        # Copyright label
        copylabel = Label(readframe,bg='Gainsboro',fg='Black',font=self.ReadFont3 \
                          ,text='Copyright (c) 2014 Daniel Kiminki',anchor=CENTER)

        # Place top frame.        
        readframe.place(x=0,y=0,width=600,height=600)

        # Place everything else
        self.helpicon_label.place(x=250,y=50,anchor=CENTER)
        readlabel.place(x=0,y=100,width=500)
        textframe.place(x=0,y=125,width=500,height=375)
        copylabel.place(x=0,y=520,width=500)
        closeb.place(x=380,y=560)
        


    # Popup window used for Check for Updates (includes icon).
    def updatewindow(self,title,updateinfo,xsize,ysize,textheight):

        # Create toplevel window
        UpdateWin = Toplevel()
        UpdateWin.geometry("%sx%s" % (xsize,ysize))
        UpdateWin.configure(background="Gainsboro")
        UpdateWin.title(title)

        # Decorative frame
        decframe = Frame(UpdateWin,relief=RAISED,border=2,bg='Gainsboro')

        # Create button to close window
        closeb = Button(UpdateWin, text="CLOSE", command=UpdateWin.destroy \
                        ,bg="Gainsboro",fg="Black" \
                        ,font=self.UpdateFont1,borderwidth=1,relief=RAISED \
                        ,highlightcolor="White",highlightbackground="Gainsboro" \
                        ,highlightthickness=1,width=7,state=NORMAL)
                     
        # Text object
        updatetext = Text(UpdateWin,width=45,wrap=WORD,relief=FLAT \
                          ,borderwidth=0,highlightthickness=0 \
                          ,background="Gainsboro",foreground="Black" \
                          ,height=textheight,font=self.UpdateFont2)
        updatetext.insert(0.0,updateinfo)
        updatetext.config(state=DISABLED)

        # JWST icon 
        self.iconimage = PhotoImage(file=self.datafilepath+'JWST_acro.gif' \
                                    ,format='gif') 
        self.updateicon_label =Tkinter.Label(UpdateWin,image = self.iconimage)
        self.updateicon_label.image=self.iconimage

        # Place things in window
        decframe.place(x=0,y=0,width=int(xsize),height=int(ysize))
        self.updateicon_label.place(x=20,y=40)
        closeb.place(x=int(xsize)-120,y=int(ysize)-40)
        updatetext.place(x=110,y=20,width=int(xsize)-140)

    #####################
    # OTHER DEFINITIONS #
    #####################

    # Reset history and hide all navigation buttons
    def hideButtons(self,event):
        self.gbb.itemconfig(self.bk,state=HIDDEN)
        self.gfb.itemconfig(self.bk,state=HIDDEN)
        self.history = []
        self.update_idletasks()
    
    # Get number of entries and update
    def NumEntries(self):
        nentries = 0
        for key in acros2:
            nentries = nentries + len(acros2[key])
        self.nEntries.delete(0,END)
        self.nEntries.insert(0,"%s entries" % nentries)

    # Action when delete button (in search box) is clicked
    def onDbClick(self,event):
        # Remove entry
        self.entry2.delete(0,END)
        # set focus on entry
        self.entry2.focus()
        # Reset history and take down forward/back/delete buttons
        self.history = []
        self.db.place_forget()
        self.gbb.place_forget()
        self.gfb.place_forget()
        self.search.place_forget()
        self.search.place(0,0)

    # Action to remove default text in search bar
    def searchDefaultTextRemove(self,event):
        if self.entry2.var.get() == '>SEARCH':
            self.entry2.delete(0,END)
            self.entry2.configure(fg="black")

    # Highlight appropriate text when searching by keyword
    def highlight(self,text_widget, keyword, tag):
        # Code for details section
        pos = '1.0'
        while True:
            idx = text_widget.search(keyword, pos, END,nocase=1)
            if not idx:
                break
            try:
                pos = '{}+{}c'.format(idx, len(keyword))
            except ValueError:
                pos = "%s+%sc" % (idx,len(keyword))
            text_widget.tag_add(tag, idx, pos)

    # ##################################
    #     Defs for links and history   #
    ####################################
    # Action when Back button is clicked
    def onGbbClick(self,event):
        # Get entry from results box
        lb_entry = self.resBox.get( \
                    self.resBox.curselection()).replace("\n","").split("->")
        current = lb_entry[0].rstrip().split("->")[0].strip()
        # Get location of entry2 in history
        try:
            curloc = self.history.index(current)
            if curloc > 0:
                back = self.history[curloc-1]
                self.entry2.delete(0,END)
                self.entry2.var.set(back)
                self.entry2.focus_set()
                # Enable forward button
                self.gfb.itemconfig(self.fw,state=NORMAL)
                self.gfb.place(x=40,y=11)
                self.update_idletasks()
            # Disable back button if at first in history
            if self.history.index(back) == 0 or curloc == 0:
                self.gbb.itemconfig(self.fw,state=HIDDEN)
                self.update_idletasks()
        except:
            pass
        
    # Action when Forward button clicked
    def onGfbClick(self,event):
        # Get entry from results box
        lb_entry = self.resBox.get( \
                    self.resBox.curselection()).replace("\n","").split("->")
        current = lb_entry[0].rstrip().split("->")[0].strip()
        # Get location of entry2 in history
        try:
            curloc = self.history.index(current)
            if (curloc+1) < len(self.history):
                forward = self.history[curloc+1]
                self.entry2.delete(0,END)
                self.entry2.var.set(forward)
                self.entry2.focus_set()
                # Enable back button
                self.gbb.itemconfig(self.bk,state=NORMAL)
                self.gbb.place(x=20,y=11)
                self.update_idletasks()
            # Disable forward button if at end of history
            if (curloc+1) == len(self.history)-1:
                self.gfb.itemconfig(self.bk,state=HIDDEN)
                self.update_idletasks()
        except:
            pass

        
    # Check details for acronyms and replace with links to 
    # that acronym. 
    def links(self,text_widget,tag):
        # Get string to search
        stuffToSearch = text_widget.get(0.0,END)
        for key in acros2.keys():
            pos = '1.0'     
            # Lots of IF statements to minimize time cost from re.
            if key not in [x.upper() for x in self.exceptions]:
                # First find out if they key is in the details.
                if key in stuffToSearch:
                    # Now determine if there if it's surrounded by whitespace
                    # or punctuation (to prevent matching problems like HS and HST).
                    match = re.findall('[\)\s.,!?\/\(]%s[\)\s.,!?\/\(]' % key \
                                       ,stuffToSearch)
                    # Now find where those matches are.
                    for item in match:
                        keyword = item.encode('ascii').strip('\s.,!?\/\)\(')
                        pos = '1.0'
                        while True:
                            idx = text_widget.search(keyword,pos,END,nocase=1)
                            if not idx:
                                break
                            try:
                                pos = '{}+{}c'.format(idx,len(keyword))
                            except ValueError:
                                pos = "%s+%sc" % (idx,len(keyword))
                            # Insert hyperlink
                            text_widget.tag_add(tag,idx,pos)
            else:
                # First find out if they key is in the details.
                newkey = self.exceptions[[x.upper() for x in self.exceptions].index(key)]
                if newkey in stuffToSearch:
                    # Now determine if there if it's surrounded by whitespace
                    # or punctuation (to prevent matching problems like HS and HST).
                    match = re.findall('[\)\s.,!?\/\(]%s[\)\s.,!?\/\(]' % newkey \
                                       ,stuffToSearch,re.IGNORECASE) 
                    # Now find where those matches are.
                    for item in match:
                        keyword = item.encode('ascii').strip('\s.,!?\/\)\(')
                        pos = '1.0'
                        while True:
                            idx = text_widget.search(keyword,pos,END,nocase=1)
                            if not idx:
                                break
                            try:
                                pos = '{}+{}c'.format(idx,len(keyword))
                            except ValueError:
                                pos = "%s+%sc" % (idx,len(keyword))
                            # Insert hyperlink
                            text_widget.tag_add(tag,idx,pos)

    # If Links are clicked in details section    
    def textClick(self,event):
        # Get what's being clicked on:
        ranges = self.detailText.tag_ranges('hyper')
        whichRange = self.detailText.index(CURRENT)
        # Have to sort through and grab the right link (if there 
        # are multiple of the same one).
        for i in range(0, len(ranges), 2):
            start = ranges[i]
            stop = ranges[i+1]
            nwhichRange = float(str(whichRange).split('.')[1])
            nstart = float(str(start).split('.')[1])
            nstop = float(str(stop).split('.')[1])
            if (nwhichRange >= nstart) and (nwhichRange <= nstop):
                # strip off all crap around match
                tagkey = self.detailText.get(start,stop).encode('ascii').strip().upper()
                # What's the current selection
                lb_entry = self.resBox.get( \
                        self.resBox.curselection()).replace("\n","").split("->")
                current = lb_entry[0].rstrip().split("->")[0].strip()
                # Add current if current is first in history
                if self.history == []:
                    self.history.append(current)
                    # Add next entry
                    self.history.append(tagkey)
                else:
                    # Remove everything after current entry first.
                    # This is in case Go back has been clicked and
                    # then another link clicked. (***the following indices
                    # code is here as a start to rectify when an 
                    # acronym is clicked on again, i.e., you go around
                    # in circles. Can't fix it completely yet***)
                    indices = [i for i, x in enumerate(self.history) if x == current]
                    self.history = self.history[0: indices[-1]+1]
                    # Now add next entry
                    self.history.append(tagkey)
                    # Deactivate forward button
                    self.gfb.place_forget()
                # Remove entry2
                self.entry2.delete(0,END)
                self.update_idletasks()
                self.entry2.var.set(tagkey)
                self.entry2.focus_set()
                # Enable go back button
                self.gbb.itemconfig(self.bk,state=NORMAL)
                self.gbb.place(x=20,y=11)
                self.update_idletasks()


    # ###################################
    # Stuff for Dictionary Modification #
    #####################################   

    # Check for version updates
    def CheckUpdate(self):
		try:
			# Now check version info
			VerUpdate = urllib2.urlopen('http://deggial.as.arizona.edu/jad/VersionInfo').read()
			if float(VerUpdate.split()[2]) > version:
				self.updatewindow("Check for Update","You are currently running version %s.\nVersion %s is now available." % (version,str(VerUpdate.split()[2])),"420","150",4)
			else:
				self.updatewindow("Check for Update","\nYou are currently running version %s.\nThere are no updates at this time." % version,"420","150",4)
		except:
			self.updatewindow("CONNECTION ERROR","\nUnable to connect to the dictionary webpage.","420","150",4)


    # Sync acronyms.dat
    def SyncAcros(self):
        try:
            update = urllib2.urlopen('http://deggial.as.arizona.edu' \
                                 +'/jad/acronyms.dat').read()
            with open(self.datafilepath+"acronyms.dat","w") as f:
                f.write(update)
                # Repopulate Dictionary
            self.PopAcroDict()
            # If successful show update (have to use 'after' for it to display right)
            self.after(50,self.nEntries.delete(0,END))
            self.after(100,self.nEntries.insert(0,"Synced"))
            # Also have to use 'update_idletasks' in conjunction with 'after' for correct display)
            self.update_idletasks()
            self.after(1000,self.NumEntries())
        # If can't connect or some other problem, then alert user
        except:
            self.updatewindow("CONNECTION ERROR","\nUnable to connect to the dictionary " \
                              +"webpage.","420","150",4)
    
    # Populate Acronym Dictionary
    def PopAcroDict(self):
        # Populate dictionary, acros2 (acros2 is holder over from when there was 
        # and acros1, and I'm too lazy to change)
        global acros2; acros2={}
        # Read acronyms.dat
        with open(self.datafilepath+"acronyms.dat","r") as f:
            data = f.read()
            paragraphs = data.split("\n\n")
            for item in paragraphs:
                try:
                    (keys,vals) = item.split("->")
                    # If key already exists, add to value list
                    if keys.upper() in acros2:
                        acros2[keys.upper()].append(vals)
                    else:
                        acros2[keys.upper()] = [vals]
                except ValueError:
                    pass

    # Write dictionary to file (!!!CURRENTLY DISABLED!!!)      
    def WriteAcroDict(self):
        #Sort dictionary and put in list of tuples.
        sortacros2=[ (k,acros2[k]) for k in sorted(acros2.keys())]
        # Open file and overwrite with modified contents.
        with open(self.datafilepath+"acronyms.dat","w") as f:
            for ii in sortacros2:
                for item in ii[1]:
                    f.write(ii[0]+'->')
                    f.write(item+'\n\n')
                    
    # If Edit is chosen from menu (!!!CURRENTLY DISABLED!!!)
    def OnEditClick(self):
        # Get entry contents
        editkey = self.resBox.get(self.resBox.curselection()).split("->")
        # Get label contents
        editlb = self.definText.get(0.0,END)
        #editlb = labelVariable.get()
        # Get output box contents
        editout = self.detailText.get(0.0,END).rstrip('\n').encode('ascii')
        # Figure out which dictionary entry this is
        matchvals = acros2[editkey[0].strip()]
        # Find out which entry in dictionary (for multiple matches)
        search = re.compile('\n'+editlb.strip())
        match = [i for i, word in enumerate(matchvals) if re.match(search,word)]
        # Change appropriate dictionary entry
        matchvals[match[0]]='\n'+editlb+' ='+editout
        acros2[editkey[0].strip()] = matchvals
        # Tell user that it has successfully been modified.
        self.definText.setText(editlb+'  has been modified')
        # Write updated dictionary file.
        self.WriteAcroDict()
        # Re-populate dictionary
        self.PopAcroDict()

    # If Add is chosen from menu (!!!CURRENTLY DISABLED!!!)
    def OnAddClick(self):
        #Get entry variable input and add to dictionary
        search = self.entry2.var.get().strip().split(' ')
        newkey = search[0].rstrip() #<=== in case user provides two words
        newval = self.detailText.get(0.0,END).encode('ascii').rstrip('\n')
        # in case user doesn't follow directions
        if newval != self.entry2.instructions and len(newval) != 0:
            if len(newkey) > 1:
                # Find out if already in dictionary
                if newkey.upper() in acros2:
                    # Append dictionary value list
                    acros2[newkey.upper()].append('\n'+newval)
                else:
                    #Add new key and value.
                    acros2[newkey.upper()]=['\n'+newval]  
                #Write updated dictionary to file.
                self.WriteAcroDict()
                #Re-populate dictionary.
                self.PopAcroDict()             
                # Update labels.
                self.definText.setText(newkey.upper()+' added to dictionary.')
                # Focus back on entry variable
                self.entry2.focus_set()
                self.entry2.var.set(newkey)
                # Clean up textbox
                self.detailText.delete(0.0,Tkinter.END)
            else:
                # Clean up textbox
                self.detailText.delete(0.0,Tkinter.END)
                # Alert user to invalidity
                self.definText.setText('Not a valid entry')
                self.detailText.insert(0.0,self.entry2.instructions)
        else:
            # Clean up textbox
            self.detailText.delete(0.0,END)
            # Alert user to invalidity 
            self.instructTextBox.place_forget()
            self.definText.setText('Not a valid entry')
            self.detailText.insert(0.0,self.entry2.instructions)

    # If delete is chosen from menu (!!!CURRENTLY DISABLED!!!)
    def OnDeleteClick(self):
        #Get entry to remove.
        entryToDel = self.definText.get(0.0,END)
        #Get all entries matching acronym.
        acroEntryToDel = self.resBox.get(ACTIVE).split("->")[0].strip()
        if acroEntryToDel in acros2:
            allEntries = acros2[acroEntryToDel]
            #If there is more than one match, remove the right one.
            if len(allEntries) > 1:
                for item in allEntries:
                    item2 = item.split('=')
                    if item2[0].strip() == entryToDel:
                        allEntries.remove(item)
                        acros2[self.entry2.var.get()]=allEntries
            #If there is only one match, remove it.
            elif len(allEntries) == 1:
                del acros2[self.resBox.get(ACTIVE).split("->")[0].strip()]
            self.definText.setText(entryToDel+' has been removed')  
        else:
            self.definText.setText('No entry for '+self.entry2.var.get())  

        #Clean up boxes.
        self.detailText.delete(0.0,Tkinter.END)

        #Write updated dictionary to file.
        self.WriteAcroDict()
        
        #Re-populate dictionary
        self.PopAcroDict()

#%%%%%%%%%%%%%%%%%%%%%%
# --SEARCHBAR CLASS-- %
#%%%%%%%%%%%%%%%%%%%%%%
class SearchBar:
    
    # Search bar definition 
    # (would be better if this was less hardcoded for position)
    def __init__(self,x1,y1,x2,y2, *args, **kwargs):
        # Use canvas to draw searchbar
        self.searchbar = Canvas(width=x2-x1, height=y2-y1,borderwidth=0 \
                                ,highlightthickness=0)
        greywidth = x2-x1
        greyheight = y2-y1
        # Gradient rectangle.
        factor = 150./greyheight
        for offset in range(0, greyheight/2):
            gradColor = '#%02x%02x%02x' % (116.+offset*factor,133.+offset*factor \
                                           ,150.+offset*factor)
            self.searchbar.create_line(x1, y1 + offset, x1+greywidth, y1 + offset \
                                       , fill=gradColor)
        for offset in range(greyheight/2, greyheight):
            gradColor = '#%02x%02x%02x' % (266.-offset*factor,283.-offset*factor \
                                           ,300.-offset*factor)
            self.searchbar.create_line(x1, y1 + offset, x1+greywidth, y1 + offset \
                                       , fill=gradColor)
        # Curved grey shapes behind entry widget
        self.searchbar.create_rectangle(x1+20,y1+6,x2-20,y1+32 \
                                        ,fill="#778899",outline="#778899")
        self.searchbar.create_oval(x1+8,y1+6,x1+32,y1+32,fill="#778899" \
                                   ,outline="#778899")
        self.searchbar.create_oval(x2-28,y1+6,x2-8,y1+32,fill="#7b8c9d" \
                                   ,outline="#778899")

        # Curved white shapes behind entry widget
        self.searchbar.create_rectangle(x1+70, y1+8, x2-20, y1+30 \
                                        , fill="white",outline="white")
        self.searchbar.create_oval(x1+60,y1+8,x1+80,y1+30,fill="white",outline="white")
        self.searchbar.create_oval(x2-30,y1+8,x2-10,y1+30,fill="white",outline="white")
        self.searchbar.place(x=x1,y=y1)

    # Added with 'place' below for redrawing searchbar
    def place_forget(self):
        self.searchbar.place_forget()

    def place(self,x,y):
        self.searchbar.place(x=x,y=y)
        
#%%%%%%%%%%%%%%%%%%%%%%%%%
# ---BETTERTEXT CLASS--- %
#%%%%%%%%%%%%%%%%%%%%%%%%%
# Lets me add tags easier
class BetterText(Text):
    def __init__(self, *args, **kwargs):

        #Initialize
        Text.__init__(self, *args, **kwargs)

        #Tags
        self.tag_configure('sometag', justify='center')

    def setText(self,newtext):
        self.delete(0.0,END)
        self.insert(0.0,newtext)
        self.tag_add('sometag',1.0,END)

#%%%%%%%%%%%%%%%%%%%%%%%%%
# --AUTOCOMPLETE CLASS-- %
#%%%%%%%%%%%%%%%%%%%%%%%%%
        
# The general layout of the autocomplete code is based on that found here:
# http://code.activestate.com/recipes/578253-an-entry-with
#-autocompletion-for-the-tkinter-gui/
        
class AutocompleteEntry(Entry):
    def __init__(self, *args, **kwargs):
        
        Entry.__init__(self, *args, **kwargs)
        self.var = self["textvariable"]
        if self.var == '':
            self.var = self["textvariable"] = StringVar()

        self.var.trace('w', self.changed)
        # Bind some events
        self.bind("<Return>",self.selection)
        self.bind("<Up>", self.up)
        self.bind("<Down>", self.down)
        # Set focus to this object
        self.focus_set()
        # Results displayed? (default = False)
        self.resBox_up = False

        # Contribute an entry instructions (display if no results)
        self.instructions="To suggest an entry, please contribute using " \
          +"the dictionary webpage:" \
          +"\n\nhttp://deggial.as.arizona/edu/jad/submit.php"

    # When some form of input is received (typed letter, delete, etc)
    def changed(self, name, index, mode): 
        # Clean up output box
        self.master.detailText.delete(0.0,END) 
        self.master.definText.delete(0.0,END)
        # What to do if searchbar has nothing or default SEARCH string
        if self.var.get() == '' or self.var.get() == '>SEARCH':
            # If listbox (results) has entries, remove them,
            # take down detailText (details) and label (definition).
            if self.resBox_up != False:
                self.master.resBox.delete(0,END)
                self.resBox_up = False                
            # Remove definition (label) and details (output)
            self.master.definText.place_forget()
            self.master.detailText.place_forget()
            # Try/Except here for startup because objects don't exist yet
            try:
                # Remove forward/back/delete buttons
                self.master.db.place_forget()
            except AttributeError:
                pass
            # Put up instructions
            self.master.instructTextBox.place_forget()
            self.update()
            self.master.instructTextBox.place(x=10,y=80,height=200)

        # If searchbar has text (not instructions text)
        else:
            # Take down instructions
            self.master.instructTextBox.place_forget()
            # Put up delete entry button
            self.master.db.place(x=310,y=12)
            # Put up detailText (details) and label (definition)
            self.master.detailText.place_forget()
            self.master.definText.place_forget()
            # Have to do an update or the above placement stuff doesn't
            # happen until after loop (i.e., things get drawn over and
            # it's not pretty).
            self.update()
            self.master.detailText.place(x=10,y=80,width=320,height=200)
            self.master.definText.place(x=10,y=10,width=320,height=65)
            # Display matches, if they exist
            words = self.comparison()
            if words:  
                self.resBox_up = True
                # Fill in matches in results listbox
                self.master.resBox.delete(0, END)
                for w in sorted(words):
                    self.master.resBox.insert(END,w)
                # See if there is an exact match and select if exist
                if self.get().upper() in acros2:
                    # grab key-definition and dust off all the extra crap.
                    exact = self.get().upper()+' -> ' \
                      +acros2[self.get().upper()][0].strip("\n").split('=')[0].rstrip(' ')
                    # get location in results listbox
                    exactloc = words.index(exact)
                    # select that location
                    self.master.resBox.selection_clear(0,END)
                    self.master.resBox.selection_set(exactloc)
                    self.master.resBox.activate(exactloc)
                    # Scroll to selection
                    self.master.resBox.yview(exactloc)
                #If no exact match, then set selection to first item.
                else:
                    self.master.resBox.selection_clear(first='0')
                    self.master.resBox.selection_set(first='0')
                    self.master.resBox.activate('0')
                # Send event to selection so that selection is displayed
                # in results
                self.selection("<Button-1>")
            else:
                # Remove results listbox entries
                self.master.resBox.delete(0,END)
                # Set listbox tracker (nothing in it now).
                self.resBox_up = False
                

    # Separate the dictionary entries 
    # (i.e., take acronym -> definition = details, and split it up)
    def SepDictComps(self,DictEntry):
        DictEntrySep = {}
        #Separate entry
        if "=" in DictEntry:
            restemp = DictEntry.split("=")
            res1 = restemp[0].strip()
            res2 = restemp[1]
            res2 = "\n".join(textwrap.wrap(res2,50))
        else:
            res1 = 'No description'
            res2 = DictEntry
        DictEntrySep[res1] = res2                
        return DictEntrySep

    # Get selection and display
    def selection(self, event):
        
        # Do hightlight of matches
        if self.get() != '':
            # For keyword search...
            if self.get()[0] == '%':
                hlightmatch = self.get().replace("%","")
            else:
                hlightmatch = self.get()

        # If autocomplete listbox has content:                        
        if self.resBox_up:
            # If there is no selected entry, then add:
            if self.master.resBox.curselection() == ():
                lb_entry = []
                lb_entry.append(self.get())
            # If there IS a selected entry:
            else:
                # Get selected entry
                lb_entry = self.master.resBox.get( \
                                self.master.resBox.curselection()).replace("\n","").split("->")
                # Place cursor in entry widget
                self.icursor(END)

            # Now, populate textbox and label
            search = lb_entry[0].rstrip().split("->")
            search_acro = search[0].strip()
            try:
                result = acros2[search_acro.upper()]
                # If focus is on entry box, choose first match
                if len(lb_entry) < 2:
                    match = 0
                # else, choose which ever is selected
                else:
                    # If two entries, get the one chosen.
                    index = 0
                    for item in result:
                        item2 = item.split('=')
                        if item2[0].strip() == lb_entry[1].strip():
                            match = index
                        index += 1
                # Separate the entry
                dictentries = self.SepDictComps(result[match])
                # Put results in label and textbox.
                self.master.detailText.place_forget()
                self.master.definText.place_forget()
                # Run 'update' of main to display right
                self.update()
                # Remove and redisplay with inserted text
                self.master.detailText.place(x=10,y=80,width=320,height=200)
                self.master.definText.place(x=10,y=10,width=320,height=65)
                self.master.definText.setText(''.join(dictentries.keys()))
                self.master.detailText.delete(0.0,Tkinter.END)
                self.master.detailText.insert(1.0
                    ,''.join(dictentries.values()).replace("\n"," "))
                
                # Highlight matches
                if highlight_onoff.get() == 1:
                    if self.get()[0] == '%':
                        self.master.highlight(self.master.detailText,hlightmatch,'highlight')
                        self.master.highlight(self.master.definText,hlightmatch,'highlight')

                # Link if links exist
                if links_onoff.get() == 1:
                    self.master.links(self.master.detailText,"hyper")

            # If selection is not in dictionary, then show how to add
            # entry.
            except KeyError:
                # If not present, let user know.
                self.master.definText.setText('Not in Dictionary')
                #Clean up textbox 
                self.master.detailText.delete(0.0,Tkinter.END)
                #Instructions on how to add
                self.master.instructTextBox.place_forget()
                self.master.detailText.insert(0.0,self.instructions)
                # Set cursor in entry widget
                self.icursor(END)
        # If list box is not up (i.e., no matches), then show
        # how to add entry.
        else:
            # If not present, let user know.
            self.master.definText.setText('Not in Dictionary')
            #Clean up textbox 
            self.master.detailText.delete(0.0,Tkinter.END)
            #Instructions on how to add
            self.master.instructTextBox.place_forget()
            self.master.detailText.insert(0.0,self.instructions)

    # If up arrow key is pressed.
    def up(self, event):
        # Do only if results box is populated
        if self.resBox_up:
            # The following is for when you reach 
            # the limits of the listbox.
            if self.master.resBox.curselection() == ():
                index = '-1'
            else:
                index = self.master.resBox.curselection()[0]
            if int(index) >= -1: 
                self.master.resBox.selection_clear(first=index)
                index = str(int(index)-1)                
                self.master.resBox.selection_set(first=index)
                self.master.resBox.activate(index) 
                self.master.resBox.yview(index)
        # this command is to prevent \ut700 from being expressed 
        #in entry widget (not sure why it happens, but it does).
        return "break"

    # If down arrow key is pressed.
    def down(self, event):
        if self.resBox_up:
            # The following is for when you reach 
            # the limits of the listbox.
            if self.master.resBox.curselection() == ():
                index = '0'                
            else:
                index = self.master.resBox.curselection()[0]
            if index != self.master.resBox.size()-1:                        
                self.master.resBox.selection_clear(first=index)
                index = str(int(index)+1)        
                self.master.resBox.selection_set(first=index)
                self.master.resBox.activate(index) 
                self.master.resBox.yview(index)
        # this command is to prevent \ut701 from being expressed 
        #in entry widget.
        return "break"

    # Get matches to the entry input (i.e., matching done as user types)
    def comparison(self):
        search_entry = self.var.get().split("->")
        # If keyword or normal search:
        if search_entry[0][0] == '%':
            # The following is too much to comment. Suffice to say, extra characters are removed,
            # things are split, things are searched for matches using 're', and then matches
            # are returned if they exist.
            search_dict = search_entry[0].replace("%","").replace("(","\(").replace(")","\)").replace(".","\.")
            matches = []
            if len(search_dict) > 0:
                for i in range(len(acros2.values())):
                    for j in range(len(acros2.values()[i])):
                        insensitive=re.compile(search_dict, re.IGNORECASE)
                        match=re.search(insensitive, acros2.values()[i][j])
                        if match:
                            match_val_split = acros2.values()[i][j].replace("\n","").split("=")
                            matches.append(acros2.keys()[i]+" -> "+match_val_split[0])
                return matches
        else:
            # Same as above, but for normal search
            search_dict = search_entry[0].replace("(","\(").replace(")","\)").replace(".","\.")
            search = re.compile('.*' + search_dict.rstrip().upper() + '.*') 
            matchlist=[]
            matches=[]
            for i in range(len(acros2.keys())):
                match = re.search(search,acros2.keys()[i])
                if match:
                    matchlist.append(acros2.keys()[i])
            for w in sorted(matchlist):
                for item in acros2[w]:
                    matches.append(w+' -> '+''.join(item).split("=")[0].strip())
            return matches


#=========> START GUI <==========
#This is executed if run from command line
if __name__=="__main__":

    # Check path and start if path and acronyms.dat exist.
    # If it does, then start gui.
    PathAndStart = CheckPath(None)
    

    

